#include <windows.h>
#include <stdio.h>
#include <TCHAR.H>

LPCTSTR MyServiceName = _T("myService");
LPCTSTR MyServiceDisplayName = _T("my service display name");

extern LPCTSTR MyServicePathName;


const unsigned int MyServiceOpCodeNumber = 7;

enum MyServiceOpCodeEnum
{
	EnumCreate=0, EnumStart=1,  EnumDelete=2, EnumStop=3, EnumContinue=4, EnumPause=5, EnumQuery=6
};


char* MyServiceOpCodeName[MyServiceOpCodeNumber] = 
{
	"create", "start",  "delete", "stop", "continue", "pause", "query"
};
 


void printServiceStatus(SERVICE_STATUS& status)
{
	printf("[service type]");
	switch (status.dwServiceType)
	{
	case SERVICE_FILE_SYSTEM_DRIVER:
		printf("The service is a file system driver.");
		break;
	case SERVICE_KERNEL_DRIVER:
		printf("The service is a device driver.");
		break;
	case SERVICE_WIN32_OWN_PROCESS:
		printf("The service runs in its own process.");
		break;
	case SERVICE_WIN32_SHARE_PROCESS:
		printf("The service shares a process with other services.");
		break;
	case SERVICE_INTERACTIVE_PROCESS:
		printf("The service can interact with the desktop.");
		break;
	}
	printf("\n");
	printf("[service state]");
	switch (status.dwCurrentState)
	{
	case SERVICE_CONTINUE_PENDING:
		printf("The service continue is pending.");
		break;
	case SERVICE_PAUSE_PENDING:
		printf("The service pause is pending.");
		break;
	case SERVICE_PAUSED:
		printf("The service is paused.");
		break;
	case SERVICE_RUNNING:
		printf("The service is running.");
		break;
	case SERVICE_START_PENDING:
		printf("The service is starting.");
		break;
	case SERVICE_STOP_PENDING:
		printf("The service is stopping.");
		break;
	case SERVICE_STOPPED:
		printf("The service is not running.");
		break;
	}
	printf("\n");
}

BOOL myServiceOperation(MyServiceOpCodeEnum opCode)
{
	SC_HANDLE scHandle  = NULL;
	SC_HANDLE svcHandle = NULL;
	BOOL result = FALSE;
	SERVICE_STATUS serviceStatus;
	if ((scHandle = OpenSCManager(NULL, SERVICES_ACTIVE_DATABASE, SC_MANAGER_ALL_ACCESS))!=  NULL)
	{
		if (opCode == EnumCreate)
		{
			if ((svcHandle = CreateService(scHandle, MyServiceName, MyServiceDisplayName, SERVICE_ALL_ACCESS, 
				SERVICE_WIN32_OWN_PROCESS, SERVICE_DEMAND_START, SERVICE_ERROR_NORMAL, MyServicePathName,
				NULL, NULL, NULL, NULL, NULL))!=NULL)
			{
				result = TRUE;				
			}
			else
			{
				_tprintf(_T("error of create service of [%d]\n"), GetLastError());
			}
		}
		else
		{
			if ((svcHandle = OpenService(scHandle, MyServiceName, SERVICE_ALL_ACCESS)) != NULL)
			{	
				switch (opCode)
				{
				case EnumDelete:
					result = DeleteService(svcHandle);						
					break;
				case EnumStart:
					result = StartService(svcHandle, 0, NULL);					
					break;					
				case EnumStop:
					result = ControlService(svcHandle, SERVICE_CONTROL_STOP, &serviceStatus);
					break;
				case EnumPause:
					result = ControlService(svcHandle, SERVICE_CONTROL_PAUSE, &serviceStatus);
					break;
				case EnumContinue:
					result = ControlService(svcHandle, SERVICE_CONTROL_CONTINUE, &serviceStatus);
					break;
				case EnumQuery:
					result = ControlService(svcHandle, SERVICE_CONTROL_INTERROGATE, &serviceStatus);
					break;				
				}
				if (result)
				{
					printServiceStatus(serviceStatus);
				}	
				else
				{
					_tprintf(_T("error of Service operation of [%d]\n"), GetLastError());
				}

			}
			else
			{
				_tprintf(_T("error of Open Service of [%d]\n"), GetLastError());
			}
		}
		if (svcHandle != NULL)
		{
			CloseServiceHandle(svcHandle);
		}
		CloseServiceHandle(scHandle);
	}
	else
	{
		_tprintf(_T("error of open sc manager of [%d]\n"), GetLastError());
	}

	return result;
}

int _tmain(int argc, LPTSTR* argv)
{
	int i;
	if (argc != 2)
	{
		_tprintf(_T("usage: "));
		for (i =0 ; i<MyServiceOpCodeNumber; i++)
		{
			_tprintf(_T("[%s]"), MyServiceOpCodeName[i]);
		}
		_tprintf(_T("\n"));
		return 0;
	}
	for (i = 0; i < MyServiceOpCodeNumber; i ++)
	{
		if (_tcsicmp(argv[1], _T(MyServiceOpCodeName[i])) == 0)
		{
			if (myServiceOperation((MyServiceOpCodeEnum)i))
			{
				_tprintf(_T("my service operation %s successful!\n"), MyServiceOpCodeName[i]);
			}
		}
	}
	return 0;
}